from factli.get_posts import posts
from factli.get_list import lists
from factli.get_leaderboard import leaderboard
import click


@click.group()
def cli():
    pass

cli.add_command(lists)
cli.add_command(posts)
cli.add_command(leaderboard)

if __name__ == "__main__":
    cli()



import os
import sys
import json
import time
import pathlib
import datetime
import requests
import pandas as pd
import click
import yagmail
import schedule
from loguru import logger

@click.command()
@click.option('--list_id', help='Saved List ID')
@click.option('--count', default=100, help='Number of posts returned per call, maximum 100, defaults to 10')
@click.option('--access_token', help='Your unique access token')
@click.option('--start_date', help='Start Date (older), Format=YYYY-MM-DD, if not given defaults to NULL')
@click.option('--end_date', help='End Date(newer), Format=YYYY-MM-DD, if not given defaults to current date')
@click.option('--time_frame', help='The interval of time to consider from the endDate. Any valid SQL interval, eg: "1 HOUR" or "30 MINUTE"')
@click.option('--log_level', help='Level of output detail (DEBUG, INFO, WARNING, ERROR). Warnings and Errors are \
              always logged in respective log-files `errors.log` and `warnings.log`.\
              Default: ERROR', default='ERROR')              
@click.option('--log_file', help='Path to logfile. Defaults to standard output.')
@click.option('--sched', help='If given, waits "sched" hour(s) and then repeats.')
@click.option('--notify', help='If given, notify email address in case of unexpected errors. Needs further setup. See README.')
@click.option('--path', help='If given, stores the output at the desired location (Absolute Path needed)')
def posts(list_id, count, access_token, start_date, end_date, log_level, log_file, sched, notify, path, time_frame):
    '''
    This function generates individual folders containing posts from
    accounts (with information) for the given List ID.

    Args:
        list_id (int): This ID corresponds to the saved list of Facebook Pages.
        count (int): The number of posts to be returned per API request.
        access_token(str): The access token associated with your CrowdTangle account.
        start_date(str): The earliest date at which a post could be posted.
        end_date(str): The latest date at which a post could be posted.

    Returns: None
    '''
    str1 = pathlib.Path.cwd()
    
    if path is not None:
        str0 = path
        os.chdir(str0)  
        
    else:
        str0 = pathlib.Path.cwd()

    
    pathlib.Path(f'results/{list_id}').mkdir(parents=True, exist_ok=True)
    os.chdir(f'{str0}/results/{list_id}')

    if access_token is None:
        import Access_Token
        access_token = Access_Token.access_token
     
    try:
        with open('last_list_saved_date.txt') as f:
            d = f.read()
            start_date = d.replace(" ", "T")
        
    except FileNotFoundError:
        logger.error("File not created with the end date")

    if end_date is None:
        d = datetime.datetime.utcnow()
        current = d - datetime.timedelta(microseconds=d.microsecond)
        current -= datetime.timedelta(days=2)
        end_date = str(current.date()) + str("T") + str(current.time())

    if log_file is None:
        logger.add(sys.stdout, level=log_level)
    else:
        logger.add(log_file, level=log_level, rotation="64 MB")
        logger.add(sys.stderr, level='ERROR')

    logger.add('errors.log', level='ERROR')
    logger.add('warnings.log', level='WARNING')

    if start_date is not None:
        query = f'https://api.crowdtangle.com/posts?token={access_token}&sortBy=oldest&listIds={list_id}&startDate={start_date}&endDate={end_date}&count={count}'
    else:
        query = f'https://api.crowdtangle.com/posts?token={access_token}&sortBy=oldest&listIds={list_id}&endDate={end_date}&timeframe={time_frame}&count={count}'

    def start_collection(query):
        logger.info(f"Starting Post Collection of {list_id}")
        
        if notify is not None:
            send_mail(notify, "Hello", f"Collection started of {list_id}", str1)
        
        @retry
        def get_page(query):

            try:

                logger.info(f"Fetching:{query}")

                r = requests.get(query, timeout=10)
                
                json_response = r.json()
                #logger.debug(json_response)
                if (json_response['status'] == 429):

                    logger.info("API rate limit hit, sleeping...")
                    time.sleep(60)
                    return query

                normalized_json = pd.json_normalize(
                    json_response['result']['posts'])
                data_frame = pd.DataFrame.from_dict(
                    normalized_json, orient="columns", dtype=str)

                posts_count = len(data_frame)
                logger.debug(posts_count)

                status = r.status_code
                logger.info(status)
                try:
                    assert status == 200
                except AssertionError as e:
                    logger.warning(r.text)
                    raise e

                if(status == 200 and posts_count != 0):

                    for i in range(posts_count):
                            
                            last_post_date = str(json_response['result']['posts'][i-1]['date'])

                            date = end_date[0:10]

                            with open(f'{date}.ndjson', 'a', encoding='utf8') as f:
                                post = json_response['result']['posts'][i-1]

                                post['written_at'] = str(datetime.datetime.now())

                                json.dump(post, f, ensure_ascii=False)
                                f.write("\n")

                            os.chdir(f'{str0}/results/{list_id}')
                    next_page_query = json_response['result']['pagination']['nextPage']

                elif posts_count == 0:
                    logger.info("No New Posts")
                    os.chdir(f'{str0}/results/{list_id}')

                    with open('last_list_saved_date.txt', 'w', encoding='utf8') as f:
                        dt = start_date.replace("T", " ")
                        f.write(dt)
                    
                    next_page_query = ''

                else:
                    logger.warning("Other Status: ", status)
                    next_page_query = ''

            except KeyError:
                logger.info(f"No next page, Collection finished for {list_id}")
                os.chdir(f'{str0}/results/{list_id}')

                with open('last_list_saved_date.txt', 'w', encoding='utf8') as f:
                    dt = start_date.replace("T", " ")
                    f.write(dt)
                
                next_page_query = ''

            return next_page_query

        while (query != ''):
            query = get_page(query)
        return None

    if sched is None:
        start_collection(query)
    else:
        logger.info(f"Scheduling job for every {sched} hour(s)")
        schedule.every(int(sched)).hours.do(start_collection, query)
        start_collection(query)
    
        while True:
            
            try:

                try: 
                    schedule.run_pending()
                    time.sleep(1)
                
                except Exception as e:
                    logger.error(e)
                    if notify is not None:
                        send_mail(notify, 'Hello', str(e), str1)

            except KeyboardInterrupt:
                logger.info("Keyboard Interrupt, Stopping Collection")
                if notify is not None:
                    send_mail(notify, "Error", "KeyboardInterrupt", str1)
                break


    return None


def retry(func):

    def retried_func(*args, **kwargs):
        max_tries = 10
        tries = 0
        total_sleep_seconds = 0

        while True:
            try:
                resp = func(*args, **kwargs)

            except (requests.exceptions.Timeout, requests.exceptions.ConnectionError, AssertionError) as e:

                logger.warning(e)

                if tries < max_tries:

                    tries += 1

                    sleep_seconds = min(((tries * 2) ** 2), max(900 - total_sleep_seconds, 30))
                    total_sleep_seconds = total_sleep_seconds + sleep_seconds
                else:
                    logger.exception('Maximum retries reached. Raising Exception …')
                    raise e

                logger.warning(f"Retry in {sleep_seconds} seconds …")
                time.sleep(sleep_seconds)
                continue

            break

        return resp

    return retried_func


def send_mail(recipient, subject, content, str1):

    try:
        yag = yagmail.SMTP(oauth2_file=str1/'gmail_creds.json')
        yag.send(recipient, subject, content)
        logger.info(f'Email sent to {recipient}.\nSubject: {subject}\n{content}')
    except Exception as e:
        logger.error(f'Sending mail failed: {e}')



import requests
import time
import pandas as pd
from datetime import date
from loguru import logger
import click


@click.command()
@click.option('--list_id', help='Saved List ID')
@click.option('--count', default=10000, help='Number of accounts returned per call, no limit')
@click.option('--access_token', help='Your unique access token')
def lists(list_id, count, access_token):
    '''
    This function generates a data frame containing all the
    accounts (with information) for the given List ID.

    Args:
        list_id (int): This ID corresponds to the saved list of Facebook pages.
        count (int): The number of accounts to be returned per API request.
        access_token(str): The access token associated with your CrowdTangle account.

    Returns: Complete data frame containing all the accounts.
                 Example(headers):   id(str), name(str), handle(str)......
    '''
    if access_token is None:
        import Access_Token
        access_token = Access_Token.access_token

    all_accounts = pd.DataFrame(columns=[''])
    query = 'https://api.crowdtangle.com/lists/' + \
        str(list_id) + '/accounts?token=' + \
        str(access_token) + '&count=' + str(count)

    while(query != ''):
        try:
            print("Fetching:\t", query)

            r = requests.get(query)
            json_response = r.json()

            normalized_json = pd.json_normalize(
                json_response['result']['accounts'])

            data_frame = pd.DataFrame.from_dict(
                normalized_json, orient="columns", dtype=str)

            pd.set_option('display.max_columns', None)
            pd.set_option('display.max_rows', None)
            pd.set_option('display.max_colwidth', None)
            accounts_count = len(data_frame)
            logger.debug(accounts_count)
            status = r.status_code

            if (status == 429):

                print("API rate limit hit, sleeping...")
                time.sleep(60)

                continue

            elif(status == 200 and accounts_count != 0):

                accounts = data_frame
                all_accounts = all_accounts.append(accounts)
                query = json_response['result']['pagination']['nextPage']

            else:
                print("Other Status: ", status)
                query = ''

        except KeyError:
            print("No next page")
            file_name = str(date.today()) + "_lists.csv"
            all_accounts.to_csv(file_name, encoding='utf-8')

            break

    return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['factli']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0',
 'loguru>=0.5.3,<0.6.0',
 'pandas>=1.2.4,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'schedule>=1.1.0,<2.0.0',
 'yagmail>=0.14.256,<0.15.0']

entry_points = \
{'console_scripts': ['factli = factli.factli:cli']}

setup_kwargs = {
    'name': 'factli',
    'version': '0.2.0',
    'description': 'A Facebook Crowdtangle Client',
    'long_description': '# DBoeS-stats\nCollectors of reputation metrics of public speakers in social media platforms \n\n### About\n\nThe Facebook directory-\n\n1. extracts posts of accounts and creates individual folders containing JSON response for every account (get_posts.py). \n2. extracts content from CrowdTangle and creates CSV files containing the statistics of facebook pages (get_list.py). \n\n### Storing API Access Token \n\nStore your API access token in a python file, name it Access_Token.py and save it in the DBoeS-stats directory.\nIn the file store the access token as:\n```\naccess_token = "API access token generated from your crowd tangle account"\n```\n## Installation\n1. Install [poetry](https://python-poetry.org/docs/#installation)\n2. Clone repository\n3. In the directory run `poetry install`\n4. Run `poetry shell` to start development virtualenv\n5. Run `factli`.\n\n### Cloning this Repo\n\nTo clone this repository type:\n\n```\ngit clone https://github.com/Leibniz-HBI/DBoeS-stats.git\n```\n### Usage\n```\nusage: factli [OPTIONS]\n\nOptions:\n  --list_id TEXT       Saved List ID\n  --count INTEGER      Number of posts returned per call, maximum 100,\n                       defaults to 10\n  --access_token TEXT  Your unique access token\n  --start_date TEXT    Start Date (older), Format=YYYY-MM-DD, if not given\n                       defaults to NULL\n  --end_date TEXT      End Date(newer), Format=YYYY-MM-DD, if not given\n                       defaults to current date\n  --time_frame TEXT    The interval of time to consider from the endDate. Any\n                       valid SQL interval, eg: "1 HOUR" or "30 MINUTE"\n  --log_level TEXT     Level of output detail (DEBUG, INFO, WARNING, ERROR).\n                       Warnings and Errors are               always logged in\n                       respective log-files `errors.log` and `warnings.log`.\n                       Default: ERROR\n  --log_file TEXT      Path to logfile. Defaults to standard output.\n  --sched TEXT         If given, waits "sched" hour(s) and then repeats.\n  --notify TEXT        If given, notify email address in case of unexpected\n                       errors. Needs further setup. See README.\n  --path TEXT          If given, stores the output at the desired location\n                       (Absolute Path needed)\n  --help               Show this message and exit.\n```\nEmail notifications with the `-n` argument use [yagmail](https://pypi.org/project/yagmail/).\n## Output\n\nOutput of get_posts.py stores the raw JSON response in the following folder structure:\n\n\n`Facebook/results/list_id/account_id/start-date_end-date.json`\n\nAn example of the JSON data can be viewed [here](https://github.com/CrowdTangle/API/wiki/Posts).\n\n## Ensure that factli is continuously running, even after restart\nIf your system can run cronjobs, stop twacapic, run `crontab -e` and add the following to your crontab:\n\n```cron\n30 6 * * *    sh -c "cd PATH/TO/YOUR/DBoeS-stats/WORKING/DIRECTORY && PATH/TO/Poetry-env run factli [YOUR ARGUMENTS HERE]" >> out.txt 2>&1\n```\n\nThis will start collection at 0630Hr (GMT) everyday. \n',
    'author': 'Aditya Kumar',
    'author_email': 'a.kumar@leibniz-hbi.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Leibniz-HBI/DBoeS-stats',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
